@@ -28,6 +28,7 @@
 
 #include <fstream>
 #include <iostream>
+#include <filesystem>
 #include <memory>
 #include <regex>
 
@@ -107,17 +108,19 @@ class RWebBrowserHandle : public RWebDisplayHandle {
    typedef pid_t browser_process_id;
 #endif
    std::string fTmpDir;         ///< temporary directory to delete at the end
+   std::string fTmpFile;        ///< temporary file to remove
    bool fHasPid{false};
    browser_process_id fPid;
 
 public:
-   RWebBrowserHandle(const std::string &url, const std::string &tmpdir, const std::string &dump) : RWebDisplayHandle(url), fTmpDir(tmpdir)
+   RWebBrowserHandle(const std::string &url, const std::string &tmpdir, const std::string &tmpfile, const std::string &dump) :
+      RWebDisplayHandle(url), fTmpDir(tmpdir), fTmpFile(tmpfile)
    {
       SetContent(dump);
    }
 
-   RWebBrowserHandle(const std::string &url, const std::string &tmpdir, browser_process_id pid)
-      : RWebDisplayHandle(url), fTmpDir(tmpdir), fHasPid(true), fPid(pid)
+   RWebBrowserHandle(const std::string &url, const std::string &tmpdir, const std::string &tmpfile, browser_process_id pid)
+      : RWebDisplayHandle(url), fTmpDir(tmpdir), fTmpFile(tmpfile), fHasPid(true), fPid(pid)
    {
    }
 
@@ -126,14 +129,16 @@ public:
 #ifdef _MSC_VER
       if (fHasPid)
          gSystem->Exec(("taskkill /F /PID "s + std::to_string(fPid) + " >NUL 2>NUL").c_str());
-      std::string rmdir = "rmdir /S /Q ";
+      std::string rmdir = "rmdir /S /Q ", rmfile = "del /F ";
 #else
       if (fHasPid)
          kill(fPid, SIGKILL);
-      std::string rmdir = "rm -rf ";
+      std::string rmdir = "rm -rf ", rmfile = "rm -f ";
 #endif
       if (!fTmpDir.empty())
          gSystem->Exec((rmdir + fTmpDir).c_str());
+      if (!fTmpFile.empty())
+         gSystem->Exec((rmfile + fTmpFile).c_str());
    }
 
 };
@@ -216,7 +221,7 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
 
    if(args.GetBrowserKind() == RWebDisplayArgs::kServer) {
       std::cout << "New web window: " << url << std::endl;
-      return std::make_unique<RWebBrowserHandle>(url, "", "");
+      return std::make_unique<RWebBrowserHandle>(url, "", "", "");
    }
 
    std::string exec;
@@ -239,8 +244,57 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
 
    ProcessGeometry(exec, args);
 
-   std::string rmdir = MakeProfile(exec, args.IsHeadless());
+   std::string rmdir = MakeProfile(exec, args.IsBatchMode() || args.IsHeadless());
 
+   std::string tmpfile;
+
+   // these are secret parameters, hide them in temp file
+   if ((url.find("token=") || url.find("key=")) && !args.IsBatchMode() && !args.IsHeadless()) {
+      TString filebase = "root_start_";
+
+      auto f = gSystem->TempFileName(filebase, nullptr, ".html");
+
+      bool ferr = false;
+
+      if (!f) {
+         ferr = true;
+      } else {
+         std::string content = std::regex_replace(
+            "<!DOCTYPE html>\n"
+            "<html lang=\"en\">\n"
+            "<head>\n"
+            "   <meta charset=\"utf-8\">\n"
+            "   <meta http-equiv=\"refresh\" content=\"0;url=$url\"/>\n"
+            "   <title>Opening ROOT widget</title>\n"
+            "</head>\n"
+            "<body>\n"
+            "<p>\n"
+            "  This page should redirect you to a ROOT widget. If it doesn't,\n"
+            "  <a href=\"$url\">click here to go to ROOT</a>.\n"
+            "</p>\n"
+            "</body>\n"
+            "</html>\n", std::regex("\\$url"), url);
+
+         if (fwrite(content.c_str(), 1, content.length(), f) != content.length())
+            ferr = true;
+
+         if (fclose(f) != 0)
+            ferr = true;
+
+         tmpfile = filebase.Data();
+
+         url = "file://"s + tmpfile;
+      }
+
+      if (ferr) {
+         if (!tmpfile.empty())
+            gSystem->Unlink(tmpfile.c_str());
+         R__LOG_ERROR(WebGUILog()) << "Fail to create temporary HTML file to startup widget";
+         return nullptr;
+      }
+   }
+
+   exec = std::regex_replace(exec, std::regex("\\$rootetcdir"), TROOT::GetEtcDir().Data());
    exec = std::regex_replace(exec, std::regex("\\$url"), url);
    exec = std::regex_replace(exec, std::regex("\\$width"), swidth);
    exec = std::regex_replace(exec, std::regex("\\$height"), sheight);
@@ -249,6 +303,8 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
 
    if (exec.compare(0,5,"fork:") == 0) {
       if (fProg.empty()) {
+         if (!tmpfile.empty())
+            gSystem->Unlink(tmpfile.c_str());
          R__LOG_ERROR(WebGUILog()) << "Fork instruction without executable";
          return nullptr;
       }
@@ -259,6 +315,8 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
 
       std::unique_ptr<TObjArray> fargs(TString(exec.c_str()).Tokenize(" "));
       if (!fargs || (fargs->GetLast()<=0)) {
+         if (!tmpfile.empty())
+            gSystem->Unlink(tmpfile.c_str());
          R__LOG_ERROR(WebGUILog()) << "Fork instruction is empty";
          return nullptr;
       }
@@ -274,17 +332,21 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
       pid_t pid;
       int status = posix_spawn(&pid, argv[0], nullptr, nullptr, argv.data(), nullptr);
       if (status != 0) {
+         if (!tmpfile.empty())
+            gSystem->Unlink(tmpfile.c_str());
          R__LOG_ERROR(WebGUILog()) << "Fail to launch " << argv[0];
          return nullptr;
       }
 
       // add processid and rm dir
 
-      return std::make_unique<RWebBrowserHandle>(url, rmdir, pid);
+      return std::make_unique<RWebBrowserHandle>(url, rmdir, tmpfile, pid);
 
 #else
 
       if (fProg.empty()) {
+         if (!tmpfile.empty())
+            gSystem->Unlink(tmpfile.c_str());
          R__LOG_ERROR(WebGUILog()) << "No Web browser found";
          return nullptr;
       }
@@ -299,12 +361,14 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
       ss >> tmp >> c >> pid;
 
       if (pid <= 0) {
+         if (!tmpfile.empty())
+            gSystem->Unlink(tmpfile.c_str());
          R__LOG_ERROR(WebGUILog()) << "Fail to launch " << fProg;
          return nullptr;
       }
 
       // add processid and rm dir
-      return std::make_unique<RWebBrowserHandle>(url, rmdir, pid);
+      return std::make_unique<RWebBrowserHandle>(url, rmdir, tmpfile, pid);
 #endif
    }
 
@@ -331,7 +395,7 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
 
       _spawnv(_P_NOWAIT, gSystem->UnixPathName(fProg.c_str()), argv.data());
 
-      return std::make_unique<RWebBrowserHandle>(url, rmdir, ""s);
+      return std::make_unique<RWebBrowserHandle>(url, rmdir, tmpfile, ""s);
    }
 
    std::string prog = "\""s + gSystem->UnixPathName(fProg.c_str()) + "\""s;
@@ -367,8 +431,7 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
       gSystem->Unlink(redirect.c_str());
    }
 
-   // add rmdir if required
-   return std::make_unique<RWebBrowserHandle>(url, rmdir, dump_content);
+   return std::make_unique<RWebBrowserHandle>(url, rmdir, tmpfile, dump_content);
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
@@ -421,7 +484,7 @@ RWebDisplayHandle::ChromeCreator::ChromeCreator(bool _edge) : BrowserCreator(tru
       fBatchExec = gEnv->GetValue((fEnvPrefix + "Batch").c_str(), "$prog --headless=new --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry $url");
       fHeadlessExec = gEnv->GetValue((fEnvPrefix + "Headless").c_str(), "$prog --headless=new --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry \'$url\' &");
    }
-   fExec = gEnv->GetValue((fEnvPrefix + "Interactive").c_str(), "$prog $geometry --new-window --app=\'$url\' &");
+   fExec = gEnv->GetValue((fEnvPrefix + "Interactive").c_str(), "$prog $geometry --new-window --app=\'$url\' >/dev/null 2>/dev/null &");
 #endif
 }
 
@@ -468,14 +531,16 @@ std::string RWebDisplayHandle::ChromeCreator::MakeProfile(std::string &exec, boo
       profile_arg = chrome_profile;
    } else {
       gRandom->SetSeed(0);
-      std::string rnd_profile = "root_chrome_profile_"s + std::to_string(gRandom->Integer(0x100000));
       profile_arg = gSystem->TempDirectory();
-
 #ifdef _MSC_VER
-      profile_arg += "\\"s + rnd_profile;
+      char slash = '\\';
 #else
-      profile_arg += "/"s + rnd_profile;
+      char slash = '/';
 #endif
+      if (!profile_arg.empty() && (profile_arg[profile_arg.length()-1] != slash))
+         profile_arg += slash;
+      profile_arg += "root_chrome_profile_"s + std::to_string(gRandom->Integer(0x100000));
+
       rmdir = profile_arg;
    }
 
@@ -508,14 +573,26 @@ RWebDisplayHandle::FirefoxCreator::FirefoxCreator() : BrowserCreator(true)
    // It gives: Invalid format. Hint: <paramlist> = <param> [, <paramlist>].
    fBatchExec = gEnv->GetValue("WebGui.FirefoxBatch", "$prog -headless -no-remote $profile $url");
    fHeadlessExec = gEnv->GetValue("WebGui.FirefoxHeadless", "$prog -headless -no-remote $profile $url &");
-   fExec = gEnv->GetValue("WebGui.FirefoxInteractive", "$prog -no-remote $profile $url &");
+   fExec = gEnv->GetValue("WebGui.FirefoxInteractive", "$prog -no-remote $profile $geometry $url &");
 #else
-   fBatchExec = gEnv->GetValue("WebGui.FirefoxBatch", "$prog --headless --private-window --no-remote $profile $url");
-   fHeadlessExec = gEnv->GetValue("WebGui.FirefoxHeadless", "fork:--headless --private-window --no-remote $profile $url");
-   fExec = gEnv->GetValue("WebGui.FirefoxInteractive", "$prog --private-window \'$url\' &");
+   fBatchExec = gEnv->GetValue("WebGui.FirefoxBatch", "$prog --headless --private-window -no-remote $profile $url");
+   fHeadlessExec = gEnv->GetValue("WebGui.FirefoxHeadless", "fork:--headless --private-window -no-remote $profile $url");
+   fExec = gEnv->GetValue("WebGui.FirefoxInteractive", "$rootetcdir/runfirefox.sh $cleanup_profile $prog -no-remote $profile $geometry -url \'$url\' &");
 #endif
 }
 
+//////////////////////////////////////////////////////////////////////////////////////////////////
+/// Process window geometry for Firefox
+
+void RWebDisplayHandle::FirefoxCreator::ProcessGeometry(std::string &exec, const RWebDisplayArgs &args)
+{
+   std::string geometry;
+   if ((args.GetWidth() > 0) && (args.GetHeight() > 0) && !args.IsHeadless())
+      geometry = "-width="s + std::to_string(args.GetWidth()) + " -height=" + std::to_string(args.GetHeight());
+
+   exec = std::regex_replace(exec, std::regex("\\$geometry"), geometry);
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////////////
 /// Create Firefox profile to run independent browser window
 
@@ -528,34 +605,64 @@ std::string RWebDisplayHandle::FirefoxCreator::MakeProfile(std::string &exec, bo
 
    const char *ff_profile = gEnv->GetValue("WebGui.FirefoxProfile", "");
    const char *ff_profilepath = gEnv->GetValue("WebGui.FirefoxProfilePath", "");
-   Int_t ff_randomprofile = gEnv->GetValue("WebGui.FirefoxRandomProfile", (Int_t) 0);
+   Int_t ff_randomprofile = gEnv->GetValue("WebGui.FirefoxRandomProfile", (Int_t) 1);
    if (ff_profile && *ff_profile) {
       profile_arg = "-P "s + ff_profile;
    } else if (ff_profilepath && *ff_profilepath) {
       profile_arg = "-profile "s + ff_profilepath;
-   } else if ((ff_randomprofile > 0) || (batch_mode && (ff_randomprofile >= 0))) {
+   } else if (ff_randomprofile > 0) {
 
       gRandom->SetSeed(0);
-      std::string rnd_profile = "root_ff_profile_"s + std::to_string(gRandom->Integer(0x100000));
       std::string profile_dir = gSystem->TempDirectory();
 
 #ifdef _MSC_VER
-      profile_dir += "\\"s + rnd_profile;
+      char slash = '\\';
 #else
-      profile_dir += "/"s + rnd_profile;
+      char slash = '/';
 #endif
+      if (!profile_dir.empty() && (profile_dir[profile_dir.length()-1] != slash))
+         profile_dir += slash;
+      profile_dir += "root_ff_profile_"s + std::to_string(gRandom->Integer(0x100000));
 
       profile_arg = "-profile "s + profile_dir;
 
       if (gSystem->mkdir(profile_dir.c_str()) == 0) {
          rmdir = profile_dir;
 
+         std::ofstream user_js(profile_dir + "/user.js", std::ios::trunc);
+         // workaround for current Firefox, without such settings it fail to close window and terminate it from batch
+         // also disable question about upload of data
+         user_js << "user_pref(\"datareporting.policy.dataSubmissionPolicyAcceptedVersion\", 2);" << std::endl;
+         user_js << "user_pref(\"datareporting.policy.dataSubmissionPolicyNotifiedTime\", \"1635760572813\");" << std::endl;
+
+         // try to ensure that window closes with last tab
+         user_js << "user_pref(\"browser.tabs.closeWindowWithLastTab\", true);" << std::endl;
+         user_js << "user_pref(\"dom.allow_scripts_to_close_windows\", true);" << std::endl;
+         user_js << "user_pref(\"browser.sessionstore.resume_from_crash\", false);" << std::endl;
+
          if (batch_mode) {
-            std::ofstream user_js(profile_dir + "/user.js", std::ios::trunc);
+            // allow to dump messages to std output
             user_js << "user_pref(\"browser.dom.window.dump.enabled\", true);" << std::endl;
-            // workaround for current Firefox, without such settings it fail to close window and terminate it from batch
-            user_js << "user_pref(\"datareporting.policy.dataSubmissionPolicyAcceptedVersion\", 2);" << std::endl;
-            user_js << "user_pref(\"datareporting.policy.dataSubmissionPolicyNotifiedTime\", \"1635760572813\");" << std::endl;
+         } else {
+            // to suppress annoying privacy tab
+            user_js << "user_pref(\"datareporting.policy.firstRunURL\", \"\");" << std::endl;
+            // to use custom userChrome.css files
+            user_js << "user_pref(\"toolkit.legacyUserProfileCustomizations.stylesheets\", true);" << std::endl;
+            // do not put tabs in title
+            user_js << "user_pref(\"browser.tabs.inTitlebar\", 0);" << std::endl;
+
+            std::ofstream times_json(profile_dir + "/times.json", std::ios::trunc);
+            times_json << "{" << std::endl;
+            times_json << "   \"created\": 1699968480952," << std::endl;
+            times_json << "   \"firstUse\": null" << std::endl;
+            times_json << "}" << std::endl;
+            if (gSystem->mkdir((profile_dir + "/chrome").c_str()) == 0) {
+               std::ofstream style(profile_dir + "/chrome/userChrome.css", std::ios::trunc);
+               // do not show tabs
+               style << "#TabsToolbar { visibility: collapse; }" << std::endl;
+               // do not show URL
+               style << "#nav-bar, #urlbar-container, #searchbar { visibility: collapse !important; }" << std::endl;
+            }
          }
 
       } else {
@@ -565,9 +672,49 @@ std::string RWebDisplayHandle::FirefoxCreator::MakeProfile(std::string &exec, bo
 
    exec = std::regex_replace(exec, std::regex("\\$profile"), profile_arg);
 
+   if (exec.find("$cleanup_profile") != std::string::npos) {
+      if (rmdir.empty()) rmdir = "<dummy>";
+      exec = std::regex_replace(exec, std::regex("\\$cleanup_profile"), rmdir);
+      rmdir.clear(); // no need to delete directory - it will be removed by script
+   }
+
    return rmdir;
 }
 
+///////////////////////////////////////////////////////////////////////////////////////////////////
+/// Check if http server required for display
+/// \param args - defines where and how to display web window
+
+bool RWebDisplayHandle::NeedHttpServer(const RWebDisplayArgs &args)
+{
+   if ((args.GetBrowserKind() == RWebDisplayArgs::kOff) || (args.GetBrowserKind() == RWebDisplayArgs::kCEF) ||
+       (args.GetBrowserKind() == RWebDisplayArgs::kQt5) || (args.GetBrowserKind() == RWebDisplayArgs::kQt6) ||
+       (args.GetBrowserKind() == RWebDisplayArgs::kLocal))
+      return false;
+
+   if (!args.IsHeadless() && (args.GetBrowserKind() == RWebDisplayArgs::kOn)) {
+
+#ifdef WITH_QT6WEB
+      auto &qt6 = FindCreator("qt6", "libROOTQt6WebDisplay");
+      if (qt6 && qt6->IsActive())
+         return false;
+#endif
+#ifdef WITH_QT5WEB
+      auto &qt5 = FindCreator("qt5", "libROOTQt5WebDisplay");
+      if (qt5 && qt5->IsActive())
+         return false;
+#endif
+#ifdef WITH_CEFWEB
+      auto &cef = FindCreator("cef", "libROOTCefDisplay");
+      if (cef && cef->IsActive())
+         return false;
+#endif
+   }
+
+   return true;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 /// Create web display
 /// \param args - defines where and how to display web window
@@ -588,18 +735,34 @@ std::unique_ptr<RWebDisplayHandle> RWebDisplayHandle::Display(const RWebDisplayA
       return handle ? true : false;
    };
 
-   if ((args.GetBrowserKind() == RWebDisplayArgs::kLocal) || (args.GetBrowserKind() == RWebDisplayArgs::kCEF)) {
-      if (try_creator(FindCreator("cef", "libROOTCefDisplay")))
+   bool handleAsLocal = (args.GetBrowserKind() == RWebDisplayArgs::kLocal) ||
+                        (!args.IsHeadless() && (args.GetBrowserKind() == RWebDisplayArgs::kOn)),
+        has_qt5web = false, has_qt6web = false, has_cefweb = false;
+
+#ifdef WITH_QT5WEB
+   has_qt5web = true;
+#endif
+
+#ifdef WITH_QT6WEB
+   has_qt6web = true;
+#endif
+
+#ifdef WITH_CEFWEB
+   has_cefweb = true;
+#endif
+
+   if ((handleAsLocal && has_qt6web) || (args.GetBrowserKind() == RWebDisplayArgs::kQt6)) {
+      if (try_creator(FindCreator("qt6", "libROOTQt6WebDisplay")))
          return handle;
    }
 
-   if ((args.GetBrowserKind() == RWebDisplayArgs::kLocal) || (args.GetBrowserKind() == RWebDisplayArgs::kQt5)) {
+   if ((handleAsLocal && has_qt5web) || (args.GetBrowserKind() == RWebDisplayArgs::kQt5)) {
       if (try_creator(FindCreator("qt5", "libROOTQt5WebDisplay")))
          return handle;
    }
 
-   if ((args.GetBrowserKind() == RWebDisplayArgs::kLocal) || (args.GetBrowserKind() == RWebDisplayArgs::kQt6)) {
-      if (try_creator(FindCreator("qt6", "libROOTQt6WebDisplay")))
+   if ((handleAsLocal && has_cefweb) || (args.GetBrowserKind() == RWebDisplayArgs::kCEF)) {
+      if (try_creator(FindCreator("cef", "libROOTCefDisplay")))
          return handle;
    }
 
@@ -608,8 +771,8 @@ std::unique_ptr<RWebDisplayHandle> RWebDisplayHandle::Display(const RWebDisplayA
       return handle;
    }
 
-   bool handleAsNative = (args.GetBrowserKind() == RWebDisplayArgs::kNative) ||
-                         (args.IsHeadless() && (args.GetBrowserKind() == RWebDisplayArgs::kDefault));
+   bool handleAsNative =
+      (args.GetBrowserKind() == RWebDisplayArgs::kNative) || (args.GetBrowserKind() == RWebDisplayArgs::kOn);
 
 #ifdef _MSC_VER
    if (handleAsNative || (args.GetBrowserKind() == RWebDisplayArgs::kEdge)) {
@@ -628,7 +791,8 @@ std::unique_ptr<RWebDisplayHandle> RWebDisplayHandle::Display(const RWebDisplayA
          return handle;
    }
 
-   if (handleAsNative || (args.GetBrowserKind() == RWebDisplayArgs::kChrome) || (args.GetBrowserKind() == RWebDisplayArgs::kFirefox) || (args.GetBrowserKind() == RWebDisplayArgs::kEdge)) {
+   if ((args.GetBrowserKind() == RWebDisplayArgs::kNative) || (args.GetBrowserKind() == RWebDisplayArgs::kChrome) ||
+       (args.GetBrowserKind() == RWebDisplayArgs::kFirefox) || (args.GetBrowserKind() == RWebDisplayArgs::kEdge)) {
       // R__LOG_ERROR(WebGUILog()) << "Neither Chrome nor Firefox browser cannot be started to provide display";
       return handle;
    }
@@ -750,28 +914,30 @@ bool RWebDisplayHandle::ProduceImages(const std::string &fname, const std::vecto
    if (jsons.empty())
       return false;
 
-   std::string _fname = fname;
-   std::transform(_fname.begin(), _fname.end(), _fname.begin(), ::tolower);
-   auto EndsWith = [&_fname](const std::string &suffix) {
+   auto EndsWith = [&fname](const std::string &suffix) {
+      std::string _fname = fname;
+      std::transform(_fname.begin(), _fname.end(), _fname.begin(), ::tolower);
       return (_fname.length() > suffix.length()) ? (0 == _fname.compare(_fname.length() - suffix.length(), suffix.length(), suffix)) : false;
    };
 
    std::vector<std::string> fnames;
 
    if (!EndsWith(".pdf")) {
-      bool has_quialifier = _fname.find("%") != std::string::npos;
+      std::string farg = fname;
+
+      bool has_quialifier = farg.find("%") != std::string::npos;
 
       if (!has_quialifier && (jsons.size() > 1)) {
-         _fname.insert(_fname.rfind("."), "%d");
+         farg.insert(farg.rfind("."), "%d");
          has_quialifier = true;
       }
 
       for (unsigned n = 0; n < jsons.size(); n++) {
          if (has_quialifier) {
-            auto expand_name = TString::Format(_fname.c_str(), (int) n);
+            auto expand_name = TString::Format(farg.c_str(), (int) n);
             fnames.emplace_back(expand_name.Data());
          } else {
-            fnames.emplace_back(_fname);
+            fnames.emplace_back(farg);
          }
       }
    }
