@@ -19,6 +19,7 @@
 #include <RooFitResult.h>
 #include "RooDataHist.h" // complete type in Binned test
 #include "RooCategory.h" // complete type in MultiBinnedConstraint test
+#include "RooNLLVar.h"   // needed in BinnedDatasetTest.VSRooNLLVar
 #include <RooFit/TestStatistics/RooUnbinnedL.h>
 #include <RooFit/TestStatistics/RooBinnedL.h>
 #include <RooFit/TestStatistics/RooSumL.h>
@@ -260,7 +261,9 @@ TEST_F(RooAbsLTest, SubEventSections)
    likelihood = RooFit::TestStatistics::buildLikelihood(pdf, data.get());
 
    auto whole = likelihood->evaluatePartition({0, 1}, 0, likelihood->getNComponents());
-   ROOT::Math::KahanSum<double> nine_parts, eleven_parts, twenty_parts;
+   ROOT::Math::KahanSum<double> nine_parts;
+   ROOT::Math::KahanSum<double> eleven_parts;
+   ROOT::Math::KahanSum<double> twenty_parts;
 
    for (std::size_t ix = 0; ix < 9; ++ix) {
       nine_parts += likelihood->evaluatePartition({static_cast<double>(ix) / 9, static_cast<double>(ix + 1) / 9}, 0,
@@ -319,3 +322,51 @@ TEST_F(SimBinnedConstrainedTest, SubEventSections)
    // See comment in BinnedDatasetTest.EventSections for explanation on why no EXPECT_EQ.
    EXPECT_DOUBLE_EQ(whole.Sum(), thrice_N_events_total_parts.Sum());
 }
+
+TEST_F(RooAbsLTest, VSRooNLLVar)
+{
+   // compare the value of the likelihood to that generated by a similarly configured RooNLLVar
+   std::tie(nll, pdf, data, values) = generate_1D_gaussian_pdf_nll(w, 10);
+   likelihood = RooFit::TestStatistics::buildLikelihood(pdf, data.get());
+
+   auto AbsL_value = likelihood->evaluatePartition({0, 1}, 0, likelihood->getNComponents());
+   auto RooNLL_value = nll->getVal();
+   EXPECT_EQ(AbsL_value.Sum(), RooNLL_value);
+}
+
+TEST_F(BinnedDatasetTest, VSRooNLLVar)
+{
+   // compare the value of the likelihood to that generated by a similarly configured RooNLLVar
+   pdf->setAttribute("BinnedLikelihood");
+   data = std::unique_ptr<RooAbsData>{pdf->generateBinned(*w.var("x"))};
+   likelihood = RooFit::TestStatistics::buildLikelihood(pdf, data.get());
+
+   // manually create NLL, ripping all relevant parts from RooAbsPdf::createNLL, except here we also set binnedL = true;
+   // this is necessary, because the RooNLLVar ctor doesn't create a binned likelihood otherwise
+   RooArgSet projDeps;
+   RooAbsTestStatistic::Configuration nll_config;
+   nll_config.verbose = false;
+   nll_config.cloneInputData = false;
+   nll_config.binnedL = true;
+   int extended = 2;
+   nll = std::make_unique<RooNLLVar>("nlletje", "-log(likelihood)", *pdf, *data, projDeps, extended, nll_config);
+
+   auto AbsL_value = likelihood->evaluatePartition({0, 1}, 0, likelihood->getNComponents());
+   auto RooNLL_value = nll->getVal();
+   EXPECT_EQ(AbsL_value.Sum(), RooNLL_value);
+}
+
+TEST_F(SimBinnedConstrainedTest, VSRooNLLVar)
+{
+   RooArgSet globalObservables{*w.var("alpha_bkg_obs_A"), *w.var("alpha_bkg_obs_B")};
+
+   // compare the value of the likelihood to that generated by a similarly configured RooNLLVar
+   likelihood = RooFit::TestStatistics::NLLFactory(*pdf, *data).GlobalObservables(globalObservables).build();
+   nll = std::unique_ptr<RooAbsReal>{pdf->createNLL(*data, RooFit::GlobalObservables(globalObservables))};
+
+   auto AbsL_value = likelihood->evaluatePartition({0, 1}, 0, likelihood->getNComponents());
+   auto RooNLL_value = nll->getVal();
+   EXPECT_EQ(AbsL_value.Sum(), RooNLL_value);
+}
+
+// TODO: add tests covering all constOptimizeTestStatistic opcode cases.
