@@ -61,16 +61,27 @@
 
 #include "xRooFitVersion.h"
 
-#include <signal.h>
+#include <csignal>
+#include "TROOT.h"
+#include "TBrowser.h"
 
-BEGIN_XROOFIT_NAMESPACE;
+BEGIN_XROOFIT_NAMESPACE
 
 std::shared_ptr<RooLinkedList> xRooFit::sDefaultNLLOptions = nullptr;
 std::shared_ptr<ROOT::Fit::FitConfig> xRooFit::sDefaultFitConfig = nullptr;
 
+const char *xRooFit::GetVersion()
+{
+   return GIT_COMMIT_HASH;
+}
+const char *xRooFit::GetVersionDate()
+{
+   return GIT_COMMIT_DATE;
+}
+
 RooCmdArg xRooFit::ReuseNLL(bool flag)
 {
-   return RooCmdArg("ReuseNLL", flag, 0, 0, 0, 0, 0, 0, 0);
+   return RooCmdArg("ReuseNLL", flag, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
 }
 
 RooCmdArg xRooFit::Tolerance(double val)
@@ -199,12 +210,14 @@ xRooFit::generateFrom(RooAbsPdf &pdf, const RooFitResult &_fr, bool expected, in
                      bool foundServer = false;
                      // note : this will work only for this type of constraints
                      // expressed as RooPoisson, RooGaussian, RooLognormal, RooGamma
+                     // SimpleGaussianConstraint is CMS's own version of a RooGaussian, which also works.
                      TClass *cClass = thePdf->IsA();
                      if (cClass != RooGaussian::Class() && cClass != RooPoisson::Class() &&
                          cClass != RooGamma::Class() && cClass != RooLognormal::Class() &&
-                         cClass != RooBifurGauss::Class()) {
+                         cClass != RooBifurGauss::Class() &&
+                         !(cClass && strcmp(cClass->GetName(), "SimpleGaussianConstraint") == 0)) {
                         TString className = (cClass) ? cClass->GetName() : "undefined";
-                        oocoutW((TObject *)0, Generation)
+                        oocoutW((TObject *)nullptr, Generation)
                            << "AsymptoticCalculator::MakeAsimovData:constraint term " << thePdf->GetName()
                            << " of type " << className << " is a non-supported type - result might be not correct "
                            << std::endl;
@@ -224,7 +237,7 @@ xRooFit::generateFrom(RooAbsPdf &pdf, const RooFitResult &_fr, bool expected, in
                         // Gamma(b, n+1, ...._ in this case n+1 is the server and we don;t have a direct dependency, but
                         // we want to set n to the b value so in case of the Gamma ignore this test
                         if (cClass != RooGamma::Class()) {
-                           oocoutE((TObject *)0, Generation)
+                           oocoutE((TObject *)nullptr, Generation)
                               << "AsymptoticCalculator::MakeAsimovData:constraint term " << thePdf->GetName()
                               << " has no direct dependence on global observable- cannot generate it " << std::endl;
                            continue;
@@ -236,48 +249,47 @@ xRooFit::generateFrom(RooAbsPdf &pdf, const RooFitResult &_fr, bool expected, in
                      // the mode of the Gamma is (k-1)*theta where theta is the inverse of the rate parameter.
                      // we assume that the global observable is defined as ngobs = k-1 and the theta parameter has the
                      // name theta otherwise we use other procedure which might be wrong
-                     RooAbsReal *thetaGamma = 0;
+                     RooAbsReal *thetaGamma = nullptr;
                      if (cClass == RooGamma::Class()) {
-                        RooFIter itc(thePdf->serverMIterator());
-                        for (RooAbsArg *a2 = itc.next(); a2 != 0; a2 = itc.next()) {
+                        for (RooAbsArg *a2 : thePdf->servers()) {
                            if (TString(a2->GetName()).Contains("theta")) {
                               thetaGamma = dynamic_cast<RooAbsReal *>(a2);
                               break;
                            }
                         }
-                        if (thetaGamma == 0) {
-                           oocoutI((TObject *)0, Generation)
+                        if (thetaGamma == nullptr) {
+                           oocoutI((TObject *)nullptr, Generation)
                               << "AsymptoticCalculator::MakeAsimovData:constraint term " << thePdf->GetName()
                               << " is a Gamma distribution and no server named theta is found. Assume that the Gamma "
                                  "scale is  1 "
                               << std::endl;
                         }
                      }
-                     RooFIter iter2(thePdf->serverMIterator());
-                     for (RooAbsArg *a2 = iter2.next(); a2 != 0; a2 = iter2.next()) {
+                     for (RooAbsArg *a2 : thePdf->servers()) {
                         RooAbsReal *rrv2 = dynamic_cast<RooAbsReal *>(a2);
                         if (rrv2 && !rrv2->dependsOn(*gob) &&
                             (!rrv2->isConstant() || !rrv2->InheritsFrom("RooConstVar"))) {
 
                            // found server not depending on the gob
                            if (foundServer) {
-                              oocoutE((TObject *)0, Generation)
+                              oocoutE((TObject *)nullptr, Generation)
                                  << "AsymptoticCalculator::MakeAsimovData:constraint term " << thePdf->GetName()
                                  << " constraint term has more server depending on nuisance- cannot generate it "
                                  << std::endl;
                               foundServer = false;
                               break;
                            }
-                           if (thetaGamma && thetaGamma->getVal() > 0)
+                           if (thetaGamma && thetaGamma->getVal() > 0) {
                               rrv.setVal(rrv2->getVal() / thetaGamma->getVal());
-                           else
+                           } else {
                               rrv.setVal(rrv2->getVal());
+                           }
                            foundServer = true;
                         }
                      }
 
                      if (!foundServer) {
-                        oocoutE((TObject *)0, Generation)
+                        oocoutE((TObject *)nullptr, Generation)
                            << "AsymptoticCalculator::MakeAsimovData - can't find nuisance for constraint term - global "
                               "observables will not be set to Asimov value "
                            << thePdf->GetName() << std::endl;
@@ -301,12 +313,13 @@ xRooFit::generateFrom(RooAbsPdf &pdf, const RooFitResult &_fr, bool expected, in
       if (auto s = dynamic_cast<RooSimultaneous *>(_pdf)) {
          // do subpdf's individually
          _obs->add(w);
-         _out.first.reset(new RooDataSet(
-            uuid, TString::Format("%s %s", _pdf->GetTitle(), (expected) ? "Expected" : "Toy"), *_obs, "weightVar"));
+         _out.first = std::make_unique<RooDataSet>(
+            uuid, TString::Format("%s %s", _pdf->GetTitle(), (expected) ? "Expected" : "Toy"), *_obs,
+            RooFit::WeightVar("weightVar"));
 
          for (auto &c : s->indexCat()) {
 #if ROOT_VERSION_CODE >= ROOT_VERSION(6, 22, 00)
-            std::string cLabel = c.first.c_str();
+            std::string cLabel = c.first;
 #else
             std::string cLabel = c->GetName();
 #endif
@@ -357,7 +370,7 @@ xRooFit::generateFrom(RooAbsPdf &pdf, const RooFitResult &_fr, bool expected, in
          _obs->add(w);
          RooArgSet _tmp;
          _tmp.add(w);
-         _out.first.reset(new RooDataSet("", "Toy", _tmp, "weightVar"));
+         _out.first = std::make_unique<RooDataSet>("", "Toy", _tmp, RooFit::WeightVar("weightVar"));
          _out.first->add(_tmp);
       } else {
          if (_pdf->canBeExtended()) {
@@ -498,11 +511,14 @@ std::shared_ptr<ROOT::Fit::FitConfig> xRooFit::defaultFitConfig()
                                             // NLL. 1 = just caching, 2 = cache and track
 #if ROOT_VERSION_CODE >= ROOT_VERSION(6, 29, 00)
    extraOpts->SetValue("StrategySequence", "0s01s12s2s3m");
-   extraOpts->SetValue("HesseStrategy", 3); // if hesse is run after minimization, will use this strategy
+   extraOpts->SetValue("HesseStrategySequence", "23");
 #else
    extraOpts->SetValue("StrategySequence", "0s01s12s2m");
-   extraOpts->SetValue("HesseStrategy", 2); // when hesse is run after minimization, will use this strategy
+   extraOpts->SetValue("HesseStrategySequence", "2");
 #endif
+   extraOpts->SetValue(
+      "HesseStrategy",
+      -1); // when hesse is run after minimization, will use this strategy. -1 means start at begin of strat sequence
    extraOpts->SetValue("LogSize", 0); // length of log to capture and save
    extraOpts->SetValue("BoundaryCheck",
                        0.); // if non-zero, warn if any post-fit value is close to boundary (e.g. 0.01 = within 1%)
@@ -516,6 +532,11 @@ std::shared_ptr<ROOT::Fit::FitConfig> xRooFit::defaultFitConfig()
    return sDefaultFitConfig;
 }
 
+ROOT::Math::IOptions *xRooFit::defaultFitConfigOptions()
+{
+   return const_cast<ROOT::Math::IOptions *>(defaultFitConfig()->MinimizerOptions().ExtraOptions());
+}
+
 class ProgressMonitor : public RooAbsReal {
 public:
    void (*oldHandlerr)(int) = nullptr;
@@ -533,14 +554,17 @@ public:
       }
    };
    ProgressMonitor(RooAbsReal &f, int interval = 30)
-      : RooAbsReal(Form("progress_%s", f.GetName()), ""), fFunc("func", "func", this, f), fInterval(interval)
+      : RooAbsReal(Form("progress_%s", f.GetName()), ""),
+        oldHandlerr(signal(SIGINT, interruptHandler)),
+        fFunc("func", "func", this, f),
+        fInterval(interval)
    {
       s.Start();
-      oldHandlerr = signal(SIGINT, interruptHandler);
+
       me = this;
       vars.reset(std::unique_ptr<RooAbsCollection>(f.getVariables())->selectByAttrib("Constant", false));
    }
-   virtual ~ProgressMonitor()
+   ~ProgressMonitor() override
    {
       if (oldHandlerr) {
          signal(SIGINT, oldHandlerr);
@@ -548,11 +572,29 @@ public:
       if (me == this)
          me = nullptr;
    };
-   ProgressMonitor(const ProgressMonitor &other, const char *name = 0)
+   ProgressMonitor(const ProgressMonitor &other, const char *name = nullptr)
       : RooAbsReal(other, name), fFunc("func", this, other.fFunc), fInterval(other.fInterval)
    {
    }
-   virtual TObject *clone(const char *newname) const override { return new ProgressMonitor(*this, newname); }
+   TObject *clone(const char *newname) const override { return new ProgressMonitor(*this, newname); }
+
+   // required forwarding methods for RooEvaluatorWrapper in 6.32 onwards
+   double defaultErrorLevel() const override { return fFunc->defaultErrorLevel(); }
+   bool getParameters(const RooArgSet *observables, RooArgSet &outputSet, bool stripDisconnected) const override
+   {
+      return fFunc->getParameters(observables, outputSet, stripDisconnected);
+   }
+   bool setData(RooAbsData &data, bool cloneData) override { return fFunc->setData(data, cloneData); }
+   double getValV(const RooArgSet *) const override { return evaluate(); }
+   void applyWeightSquared(bool flag) override { fFunc->applyWeightSquared(flag); }
+   void printMultiline(std::ostream &os, Int_t contents, bool verbose = false, TString indent = "") const override
+   {
+      fFunc->printMultiline(os, contents, verbose, indent);
+   }
+   void constOptimizeTestStatistic(ConstOpCode opcode, bool doAlsoTrackingOpt) override
+   {
+      fFunc->constOptimizeTestStatistic(opcode, doAlsoTrackingOpt);
+   }
 
    double evaluate() const override
    {
@@ -577,12 +619,25 @@ public:
       if (s.RealTime() > fInterval) {
          double evalRate = (counter - prevCounter) / s.RealTime();
          s.Reset();
-         std::cerr << (counter) << ") (" << evalRate << "Hz) " << TDatime().AsString();
+         std::stringstream sout;
+
+         sout << (counter) << ") (" << evalRate << "Hz) " << TDatime().AsString();
          if (!fState.empty())
-            std::cerr << " : " << fState;
-         std::cerr << " : " << minVal << " Delta = " << (minVal - prevMin);
+            sout << " : " << fState;
+         if (counter2) {
+            // doing a hesse step, estimate progress based on evaluations
+            int nRequired = prevPars.size();
+            if (nRequired > 1) {
+               nRequired *= (nRequired - 1) / 2;
+               if (fState == "Hesse3") {
+                  nRequired *= 4;
+               }
+               sout << " (~" << int(100.0 * (counter - counter2) / nRequired) << "%)";
+            }
+         }
+         sout << " : " << minVal << " Delta = " << (minVal - prevMin);
          if (minVal < prevMin) {
-            std::cerr << " : ";
+            sout << " : ";
             // compare minPars and prevPars, print biggest deltas
             std::vector<std::pair<double, std::string>> parDeltas;
             parDeltas.reserve(minPars.size());
@@ -597,16 +652,36 @@ public:
                if (parDeltas.at(i).first == 0)
                   break;
                if (i != 0)
-                  std::cerr << ",";
-               std::cerr << parDeltas.at(i).second << (parDeltas.at(i).first >= 0 ? "+" : "-") << "="
-                         << std::abs(parDeltas.at(i).first) << "("
-                         << minPars.getRealValue(parDeltas.at(i).second.c_str()) << ")";
+                  sout << ",";
+               sout << parDeltas.at(i).second << (parDeltas.at(i).first >= 0 ? "+" : "-") << "="
+                    << std::abs(parDeltas.at(i).first) << "(" << minPars.getRealValue(parDeltas.at(i).second.c_str())
+                    << ")";
             }
             if (i < int(parDeltas.size()) && parDeltas.at(i).first != 0)
-               std::cerr << " ...";
+               sout << " ...";
             prevPars.assignFast(minPars);
          }
-         std::cerr << std::endl;
+
+         if (gROOT->FromPopUp() && gROOT->GetListOfBrowsers()->At(0)) {
+            auto browser = dynamic_cast<TBrowser *>(gROOT->GetListOfBrowsers()->At(0));
+            std::string status = sout.str();
+            int col = 0;
+            while (col < 4) {
+               std::string status_part;
+               if (status.find(" : ") != std::string::npos) {
+                  status_part = status.substr(0, status.find(" : "));
+                  status = status.substr(status.find(" : ") + 3);
+               } else {
+                  status_part = status;
+                  status = "";
+               }
+               browser->SetStatusText(status_part.c_str(), col);
+               col++;
+            }
+            gSystem->ProcessEvents();
+         }
+         std::cerr << sout.str() << std::endl;
+
          prevMin = minVal;
          prevCounter = counter;
       } else {
@@ -616,10 +691,11 @@ public:
    }
 
    std::string fState;
+   mutable int counter = 0;
+   int counter2 = 0; // used to estimate progress of a Hesse calculation
 
 private:
    RooRealProxy fFunc;
-   mutable int counter = 0;
    mutable double minVal = std::numeric_limits<double>::infinity();
    mutable double prevMin = std::numeric_limits<double>::infinity();
    mutable RooArgList minPars;
@@ -637,10 +713,7 @@ xRooFit::StoredFitResult::StoredFitResult(RooFitResult *_fr) : TNamed(*_fr)
    fr.reset(_fr);
 }
 
-xRooFit::StoredFitResult::StoredFitResult(const std::shared_ptr<RooFitResult> &_fr) : TNamed(*_fr)
-{
-   fr = _fr;
-}
+xRooFit::StoredFitResult::StoredFitResult(const std::shared_ptr<RooFitResult> &_fr) : TNamed(*_fr), fr(_fr) {}
 
 std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
                                                       const std::shared_ptr<ROOT::Fit::FitConfig> &_fitConfig,
@@ -664,10 +737,11 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       while (st.NextToken()) {
          TString parName = st;
          TString parVal = nll.getStringAttribute(parName);
-         if (parVal.IsFloat())
+         if (parVal.IsFloat()) {
             fUserPars.addClone(RooRealVar(parName, parName, parVal.Atof()));
-         else
+         } else {
             fUserPars.addClone(RooStringVar(parName, parName, parVal));
+         }
       }
    }
 
@@ -680,6 +754,7 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
    int _progress = 0;
    double boundaryCheck = 0;
    std::string s;
+   std::string hs;
    int logSize = 0;
 #if ROOT_VERSION_CODE >= ROOT_VERSION(6, 29, 00)
    int hesseStrategy = 3; // uses most precise hesse settings (step sizes and g2 tolerances)
@@ -692,8 +767,10 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       fitConfig.MinimizerOptions().ExtraOptions()->GetRealValue("BoundaryCheck", boundaryCheck);
       fitConfig.MinimizerOptions().ExtraOptions()->GetIntValue("LogSize", logSize);
       fitConfig.MinimizerOptions().ExtraOptions()->GetIntValue("HesseStrategy", hesseStrategy);
+      fitConfig.MinimizerOptions().ExtraOptions()->GetNamedValue("HesseStrategySequence", hs);
    }
    TString m_strategy = s;
+   TString m_hessestrategy = hs;
 
    // if fit caching enabled, try to locate a valid fitResult
    // must have matching constPars
@@ -703,7 +780,7 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       if (auto nllDir = cacheDir->GetDirectory(nll.GetName()); nllDir) {
          if (auto keys = nllDir->GetListOfKeys(); keys) {
             for (auto &&k : *keys) {
-               auto cl = TClass::GetClass(((TKey *)k)->GetClassName());
+               auto cl = TClass::GetClass((static_cast<TKey *>(k))->GetClassName());
                if (cl->InheritsFrom("RooFitResult")) {
                   StoredFitResult *storedFr =
                      nllDir->GetList() ? dynamic_cast<StoredFitResult *>(nllDir->GetList()->FindObject(k->GetName()))
@@ -724,8 +801,18 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
                         for (auto &p : *constPars) {
                            auto v = dynamic_cast<RooAbsReal *>(p);
                            if (!v) {
-                              match = false;
-                              break;
+                              if (auto c = dynamic_cast<RooAbsCategory *>(p)) {
+                                 if (auto _p =
+                                        dynamic_cast<RooAbsCategory *>(cachedFit->constPars().find(p->GetName()));
+                                     _p && !_p->getAttribute("global") &&
+                                     _p->getCurrentIndex() != c->getCurrentIndex()) {
+                                    match = false;
+                                    break;
+                                 }
+                              } else {
+                                 match = false;
+                                 break;
+                              }
                            };
                            if (auto _p = dynamic_cast<RooAbsReal *>(cachedFit->constPars().find(p->GetName())); _p) {
                               // note: do not need global observable values to match (globals currently added to
@@ -776,10 +863,10 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       TMatrixDSym d;
       d.ResizeTo(parsList.size(), parsList.size());
       result->setCovarianceMatrix(d);
-      result->setCovQual(-1);
+      result->setCovQual(floatPars->empty() ? 3 : -1);
       result->setMinNLL(_nll->getVal());
       result->setEDM(0);
-      result->setStatus(floatPars->getSize() == 0 ? 0 : 1);
+      result->setStatus(floatPars->empty() ? 0 : 1);
 
       std::vector<std::pair<std::string, int>> statusHistory;
       statusHistory.emplace_back(std::make_pair("EVAL", result->status()));
@@ -922,10 +1009,11 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       TString minim = _minimizer.fitter()->Config().MinimizerType();
       TString algo = _minimizer.fitter()->Config().MinimizerAlgoType();
       if (minim == "Minuit2") {
-         if (strategy == -1)
+         if (strategy == -1) {
             sIdx = 0;
-         else
+         } else {
             sIdx = m_strategy.Index('0' + strategy);
+         }
          if (sIdx == -1) {
             Warning("minimize", "Strategy %d not specified in StrategySequence %s ... defaulting to start of sequence",
                     strategy, m_strategy.Data());
@@ -943,6 +1031,8 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
             algo = "migradImproved";
          } else if (m_strategy(sIdx) == 's') {
             algo = "Scan";
+         } else if (m_strategy(sIdx) == 'h') {
+            break; // jumping straight to a hesse evaluation
          } else {
             strategy = int(m_strategy(sIdx) - '0');
             _minimizer.setStrategy(strategy);
@@ -980,9 +1070,10 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
             break; // fit was good
 
          if (status == 4 && minim != "Minuit") {
-            if (printLevel >= -1)
+            if (printLevel >= -1) {
                Warning("fitTo", "%s Hit max function calls of %d", fitName.Data(),
                        _minimizer.fitter()->Config().MinimizerOptions().MaxFunctionCalls());
+            }
             if (autoMaxCalls) {
                if (printLevel >= -1)
                   Warning("fitTo", "will try doubling this");
@@ -997,12 +1088,13 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
          // NOTE: minuit2 seems to distort the tolerance in a weird way, so that tol becomes 1000 times smaller than
          // specified Also note that if fits are failing because of edm over max, it can be a good idea to activate the
          // Offset option when building nll
-         if (printLevel >= -1)
+         if (printLevel >= -1) {
             Warning("fitTo", "%s %s%s Status=%d (edm=%f, tol=%f, strat=%d), tries=#%d...", fitName.Data(),
                     _minimizer.fitter()->Config().MinimizerType().c_str(),
                     _minimizer.fitter()->Config().MinimizerAlgoType().c_str(), status,
                     _minimizer.fitter()->Result().Edm(), _minimizer.fitter()->Config().MinimizerOptions().Tolerance(),
                     _minimizer.fitter()->Config().MinimizerOptions().Strategy(), tries);
+         }
 
          // decide what to do next based on strategy sequence
          if (sIdx == m_strategy.Length() - 1) {
@@ -1031,7 +1123,7 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       // currently dont have a way to access the covariance "dcovar" which is a metric from iterative
       // covariance method that is used by minuit2 to say if the covariance is accurate or not
       // See MinimumError.h: IsAccurate if Dcovar < 0.1
-      // Note that if strategy=2 or strategy=1 and Dcovar>0.05 then hesse will be forced to be run (see
+      // Note that if strategy>=2 or (strategy=1 and Dcovar>0.05) then hesse will be forced to be run (see
       // VariadicMetricBuilder) So only in Strategy=0 can you skip hesse (even if SetParabErrors false).
 
       double dCovar = std::numeric_limits<double>::quiet_NaN();
@@ -1040,10 +1132,11 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       //    dCovar = _minuit2->fMinimum->Error().Dcovar();
       // }
 
-      // only do hesse if was a valid min and not strat2 or above (since such strat already ran hesse, albeit with
-      // allowing for forced pos-def) or if requested hesse strategy is different to the strategy that minimization ran
-      // at
-      if (hesse && (strategy < 2 || strategy != hesseStrategy) && _minimizer.fitter()->Result().IsValid()) {
+      // only do hesse if was a valid min and not full accurate cov matrix already (can happen if e.g. ran strat2)
+      if (hesse &&
+          (m_strategy(sIdx) == 'h' || ((strategy < 2 || _minimizer.fitter()->GetMinimizer()->CovMatrixStatus() != 3) &&
+                                       _minimizer.fitter()->Result().IsValid()))) {
+
          // Note: minima where the covariance was made posdef are deemed 'valid' ...
 
          // remove limits on pars before calculation - CURRENTLY HAS NO EFFECT, minuit still holds the state as
@@ -1064,53 +1157,83 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
          // std::cout << "nIterations = " << _minimizer.fitter()->GetMinimizer()->NIterations() << std::endl;
          // std::cout << "covQual before hesse = " << _minimizer.fitter()->GetMinimizer()->CovMatrixStatus() <<
          // std::endl;
-
-         _minimizer.fitter()->Config().MinimizerOptions().SetStrategy(hesseStrategy);
-
-         // const_cast<ROOT::Math::IOptions*>(_minimizer.fitter()->Config().MinimizerOptions().ExtraOptions())->SetValue("HessianStepTolerance",0.1);
-         // const_cast<ROOT::Math::IOptions*>(_minimizer.fitter()->Config().MinimizerOptions().ExtraOptions())->SetValue("HessianG2Tolerance",0.02);
-
-         if (auto fff = dynamic_cast<ProgressMonitor *>(_nll); fff) {
-            fff->fState = TString::Format("Hesse%d", _minimizer.fitter()->Config().MinimizerOptions().Strategy());
+         sIdx = -1;
+         if (hesseStrategy == -1) {
+            sIdx = 0;
+         } else {
+            sIdx = m_hessestrategy.Index('0' + hesseStrategy);
          }
+         if (sIdx == -1) {
+            Warning("minimize",
+                    "HesseStrategy %d not specified in HesseStrategySequence %s ... defaulting to start of sequence",
+                    hesseStrategy, m_hessestrategy.Data());
+            sIdx = 0;
+         }
+         while (sIdx != -1) {
+            hesseStrategy = int(m_hessestrategy(sIdx) - '0');
+            _minimizer.fitter()->Config().MinimizerOptions().SetStrategy(hesseStrategy);
+            // const_cast<ROOT::Math::IOptions*>(_minimizer.fitter()->Config().MinimizerOptions().ExtraOptions())->SetValue("HessianStepTolerance",0.1);
+            // const_cast<ROOT::Math::IOptions*>(_minimizer.fitter()->Config().MinimizerOptions().ExtraOptions())->SetValue("HessianG2Tolerance",0.02);
 
-         //_nll->getVal(); // for reasons I dont understand, if nll evaluated before hesse call the edm is smaller? -
-         // and also becomes WRONG :-S
-
-         // auto _status = (_minimizer.fitter()->CalculateHessErrors()) ? _minimizer.fitter()->Result().Status() : -1;
-         auto _status = _minimizer.hesse(); // note: I have seen that you can get 'full covariance quality' without
-                                            // running hesse ... is that expected?
-         // note: hesse status will be -1 if hesse failed (no covariance matrix)
-         // otherwise the status appears to be whatever was the status before
-         // note that hesse succeeds even if the cov matrix it calculates is forced pos def. Failure is only
-         // if it cannot calculate a cov matrix at all.
-         if (_status != -1)
-            _status = 0; // mark as hesse succeeded, although need to look at covQual to see if was any good
-
-         /*for(auto f : *floatPars) {
-            auto v = dynamic_cast<RooRealVar*>(f);
-            if(v->hasRange("backup")) {
-               v->setRange(v->getMin(),v->getMax());
-               v->removeRange("backup");
+            if (auto fff = dynamic_cast<ProgressMonitor *>(_nll); fff) {
+               fff->fState = TString::Format("Hesse%d", _minimizer.fitter()->Config().MinimizerOptions().Strategy());
+               fff->counter2 = fff->counter;
             }
-         }
-         _minimizer.fitter()->Config().SetParamsSettings(parSettings);*/
 
-         /*for (auto &ss : _minimizer.fitter()->Config().ParamsSettings()) {
-            if( ss.HasLowerLimit() || ss.HasUpperLimit() ) std::cout << ss.Name() << " limit restored " <<
-         ss.LowerLimit() << " - " << ss.UpperLimit() << std::endl;
-         }*/
+            //_nll->getVal(); // for reasons I dont understand, if nll evaluated before hesse call the edm is smaller? -
+            // and also becomes WRONG :-S
 
-         statusHistory.push_back(std::pair<std::string, int>(
-            TString::Format("Hesse%d", _minimizer.fitter()->Config().MinimizerOptions().Strategy()), _status));
+            // auto _status = (_minimizer.fitter()->CalculateHessErrors()) ? _minimizer.fitter()->Result().Status() :
+            // -1;
+            auto _status = _minimizer.hesse(); // note: I have seen that you can get 'full covariance quality' without
+                                               // running hesse ... is that expected?
+            // note: hesse status will be -1 if hesse failed (no covariance matrix)
+            // otherwise the status appears to be whatever was the status before
+            // note that hesse succeeds even if the cov matrix it calculates is forced pos def. Failure is only
+            // if it cannot calculate a cov matrix at all.
+            if (_status != -1)
+               _status = 0; // mark as hesse succeeded, although need to look at covQual to see if was any good
 
-         if (auto fff = dynamic_cast<ProgressMonitor *>(_nll); fff && fff->fInterrupt) {
-            delete _nll;
-            throw std::runtime_error("Keyboard interrupt while hesse calculating");
-         }
-         if (_status != 0 && status == 0 && printLevel >= -1) {
-            Warning("fitTo", "%s hesse status is %d", fitName.Data(), _status);
-         }
+            /*for(auto f : *floatPars) {
+               auto v = dynamic_cast<RooRealVar*>(f);
+               if(v->hasRange("backup")) {
+                  v->setRange(v->getMin(),v->getMax());
+                  v->removeRange("backup");
+               }
+            }
+            _minimizer.fitter()->Config().SetParamsSettings(parSettings);*/
+
+            /*for (auto &ss : _minimizer.fitter()->Config().ParamsSettings()) {
+               if( ss.HasLowerLimit() || ss.HasUpperLimit() ) std::cout << ss.Name() << " limit restored " <<
+            ss.LowerLimit() << " - " << ss.UpperLimit() << std::endl;
+            }*/
+
+            statusHistory.push_back(std::pair<std::string, int>(
+               TString::Format("Hesse%d", _minimizer.fitter()->Config().MinimizerOptions().Strategy()), _status));
+
+            if (auto fff = dynamic_cast<ProgressMonitor *>(_nll); fff && fff->fInterrupt) {
+               delete _nll;
+               throw std::runtime_error("Keyboard interrupt while hesse calculating");
+            }
+            if ((_status != 0 || _minimizer.fitter()->GetMinimizer()->CovMatrixStatus() != 3) && status == 0 &&
+                printLevel >= -1) {
+               Warning("fitTo", "%s hesse status is %d, covQual=%d", fitName.Data(), _status,
+                       _minimizer.fitter()->GetMinimizer()->CovMatrixStatus());
+            }
+
+            if (sIdx >= m_hessestrategy.Length() - 1) {
+               break; // run out of strategies to try, stop
+            }
+
+            if (_status == 0 && _minimizer.fitter()->GetMinimizer()->CovMatrixStatus() == 3) {
+               // covariance is valid!
+               break;
+            } else if (_status == 0) {
+               // set the statusHistory to the cov status, since that's more informative
+               statusHistory.back().second = _minimizer.fitter()->GetMinimizer()->CovMatrixStatus();
+            }
+            sIdx++;
+         } // end of hesse attempt loop
       }
 
       // call minos if requested on any parameters
@@ -1155,12 +1278,9 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
       if (boundaryCheck) {
          // check if any of the parameters are at their limits (potentially a problem with fit)
          // or their errors go over their limits (just a warning)
-         RooFIter itr = floatPars->fwdIterator();
-         RooAbsArg *a = 0;
          int limit_status = 0;
          std::string listpars;
-         while ((a = itr.next())) {
-            RooRealVar *v = dynamic_cast<RooRealVar *>(a);
+         for (auto *v : dynamic_range_cast<RooRealVar *>(*floatPars)) {
             if (!v)
                continue;
             double vRange = v->getMax() - v->getMin();
@@ -1196,9 +1316,10 @@ std::shared_ptr<const RooFitResult> xRooFit::minimize(RooAbsReal &nll,
             }
          }
          if (limit_status == 900) {
-            if (printLevel >= 0)
+            if (printLevel >= 0) {
                Warning("minimize", "BOUNDCHK: Parameters within %g%% limit in fit result: %s", boundaryCheck * 100,
                        listpars.c_str());
+            }
          } else if (limit_status > 0) {
             if (printLevel >= 0)
                Warning("minimize", "BOUNDCHK: Parameters near limit in fit result");
@@ -1362,7 +1483,8 @@ int xRooFit::minos(RooAbsReal &nll, const RooFitResult &ufit, const char *parNam
       double val_pre =
          val_guess -
          10 * precision * sigma_guess; // this is just to set value st. guarantees will do at least one iteration
-      bool lastOverflow = false, lastUnderflow = false;
+      bool lastOverflow = false;
+      bool lastUnderflow = false;
       while (std::abs(val_pre - val_guess) > precision * sigma_guess) {
          val_pre = val_guess;
          if (val_guess > 0 && par->getMax() < val_guess)
@@ -1507,10 +1629,11 @@ xRooFit::hypoTest(RooWorkspace &w, int nToysNull, int /*nToysAlt*/, const xRooFi
       if (p->hasClients())
          continue;
       flagCount += p->getAttribute("hypoTest");
-      if (p->getAttribute("hypoTest"))
+      if (p->getAttribute("hypoTest")) {
          topPdfs.push_front(p);
-      else
+      } else {
          topPdfs.push_back(p);
+      }
    }
    if (topPdfs.empty()) {
       Error("hypoTest", "Cannot find top-level pdf in workspace");
@@ -1605,11 +1728,12 @@ xRooFit::hypoTest(RooWorkspace &w, int nToysNull, int /*nToysAlt*/, const xRooFi
          altVal = mu->getMin("physical");
          Info("hypoTest", "No altVal specified - using min of given physical range = %g", altVal);
       } else {
-         if (!std::isnan(altVal))
+         if (!std::isnan(altVal)) {
             Info("hypoTest", "alt hypo: %g - CLs activated", altVal);
-         else
+         } else {
             Info("hypoTest", "No altVal found - to specify setStringAttribute(\"altVal\",\"<value>\") on POI or set "
                              "the physical range");
+         }
       }
       bool doCLs = !std::isnan(altVal) && std::abs(mu->getMin("hypoPoints")) > altVal &&
                    std::abs(mu->getMax("hypoPoints")) > altVal;
@@ -1628,7 +1752,8 @@ xRooFit::hypoTest(RooWorkspace &w, int nToysNull, int /*nToysAlt*/, const xRooFi
       std::vector<int> expSig = {-2, -1, 0, 1, 2};
       if (std::isnan(altVal))
          expSig.clear();
-      std::map<int, TGraphErrors> exp_pcls, exp_cls;
+      std::map<int, TGraphErrors> exp_pcls;
+      std::map<int, TGraphErrors> exp_cls;
       for (auto &s : expSig) {
          exp_pcls[s].SetNameTitle(TString::Format("exp%d_p%s", s, sCL),
                                   TString::Format("Expected (%d#sigma) p_{%s};%s", s, sCL, mu->GetTitle()));
@@ -1667,13 +1792,14 @@ xRooFit::hypoTest(RooWorkspace &w, int nToysNull, int /*nToysAlt*/, const xRooFi
             exp_pcls[s].SetPoint(exp_pcls[s].GetN(), testVal,
                                  (doCLs) ? hp.pCLs_asymp(s).first : hp.pNull_asymp(s).first);
          }
-         if (doCLs)
+         if (doCLs) {
             Info("hypoTest", "%s=%g: %s=%g sigma_mu=%g %s=%g", mu->GetName(), testVal, obs_ts->GetName(),
                  obs_ts->GetPointY(obs_ts->GetN() - 1), hp.sigma_mu().first, obs_pcls->GetName(),
                  obs_pcls->GetPointY(obs_pcls->GetN() - 1));
-         else
+         } else {
             Info("hypoTest", "%s=%g: %s=%g %s=%g", mu->GetName(), testVal, obs_ts->GetName(),
                  obs_ts->GetPointY(obs_ts->GetN() - 1), obs_pcls->GetName(), obs_pcls->GetPointY(obs_pcls->GetN() - 1));
+         }
       };
 
       if (mu->getBins("hypoPoints") <= 0) {
@@ -1752,7 +1878,7 @@ xRooFit::hypoTest(RooWorkspace &w, int nToysNull, int /*nToysAlt*/, const xRooFi
          band2->SetBit(kCanDelete);
          band2up->SetBit(kCanDelete);
          band2down->SetBit(kCanDelete);
-         auto ax = (TNamed *)band2->Clone(".axis");
+         auto ax = static_cast<TNamed *>(band2->Clone(".axis"));
          ax->SetTitle(TString::Format("Hypothesis Test;%s", mu->GetTitle()));
          ax->Draw("AF");
          band2->Draw("F");
@@ -1846,7 +1972,7 @@ double round_to_digits(double value, int digits)
       return 0.0;
    double factor = pow(10.0, digits - ceil(log10(std::abs(value))));
    return std::round(value * factor) / factor;
-};
+}
 double round_to_decimal(double value, int decimal_places)
 {
    const double multiplier = std::pow(10.0, decimal_places);
@@ -1871,4 +1997,4 @@ std::pair<double, double> xRooFit::matchPrecision(const std::pair<double, double
    return out;
 }
 
-END_XROOFIT_NAMESPACE;
+END_XROOFIT_NAMESPACE
